var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { useState, useEffect } from "react";
import { storeManager } from "./utils/store-manager";
// Global Store and Subscribers
const store = {};
const subscribers = {};
// Default Middleware
const defaultLogger = (state, updater, next) => {
    console.log("🔄 Previous State:", state);
    next(updater);
    console.log("✅ Updated State:", state);
};
const defaultAsync = (state, updater, next) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const result = updater(state);
        if (result !== undefined &&
            result !== null &&
            typeof result === "object" &&
            typeof result.then === "function") {
            console.log("⏳ Async Action Started");
            yield result;
            console.log("✅ Async Action Completed");
        }
        next(updater); // Ensure next is called once
    }
    catch (error) {
        console.error("❌ Async Middleware Error:", error);
        next(() => { }); // Safely move to the next middleware
    }
});
// 🛠️ Core State Management Function with Redux DevTools Support
export function createState(// 🔥 Enforce T as an object
name, initialState, middlewares = [], registerToGlobalStore = true) {
    var _a, _b;
    if (store[name])
        throw new Error(`State slice "${name}" already exists.`);
    store[name] = initialState;
    subscribers[name] = new Set();
    const combinedMiddlewares = [defaultLogger, defaultAsync, ...middlewares];
    // 🛠️ Redux DevTools Integration
    const devTools = typeof window !== "undefined" &&
        ((_b = (_a = window.__REDUX_DEVTOOLS_EXTENSION__) === null || _a === void 0 ? void 0 : _a.connect) === null || _b === void 0 ? void 0 : _b.call(_a));
    if (devTools) {
        devTools.init(initialState);
        console.log(`🛠️ Redux DevTools initialized for slice: "${name}"`);
    }
    const applyMiddleware = (updater, actionType = "STATE_UPDATE") => {
        let index = 0;
        const next = (nextUpdater) => {
            if (index < combinedMiddlewares.length) {
                const currentMiddleware = combinedMiddlewares[index++];
                currentMiddleware(store[name], nextUpdater, next);
            }
            else {
                // Final updater application
                nextUpdater(store[name]);
                // Notify Redux DevTools
                if (devTools) {
                    devTools.send(`${name}/${actionType}`, store[name]);
                }
                // Notify subscribers only once
                const seenCallbacks = new Set();
                subscribers[name].forEach((callback) => {
                    if (!seenCallbacks.has(callback)) {
                        seenCallbacks.add(callback);
                        callback();
                    }
                });
            }
        };
        next(updater);
    };
    const get = () => store[name];
    // const set = (
    //   updater: (state: T) => void,
    //   actionType: string = "STATE_UPDATE"
    // ) => {
    //   console.log(`🛠️ set called with actionType: ${actionType}`);
    //   const prevState = { ...store[name] };
    //   applyMiddleware(updater, actionType);
    //   // Update global state and notify global subscribers
    //   if (registerToGlobalStore) {
    //     storeManager.registerSlice({
    //       name,
    //       getState: get,
    //       setState: set,
    //     });
    //   }
    //   console.log(`🔄 Previous State:`, prevState);
    //   console.log(`✅ Updated State:`, store[name]);
    // };
    const set = (updater, actionType = "STATE_UPDATE") => {
        const prevState = Object.assign({}, store[name]);
        applyMiddleware((state) => {
            const newState = Object.assign({}, state);
            updater(newState);
            if (JSON.stringify(prevState) !== JSON.stringify(newState)) {
                console.log(`🛠️ State change detected for ${name}`);
                Object.assign(state, newState); // 🔥 Correctly apply state updates
            }
        }, actionType);
        storeManager.updateGlobalState(); // 🔥 Ensure state propagates properly
    };
    // Register the slice to the StoreManager if enabled (only once during initialization)
    if (registerToGlobalStore) {
        storeManager.registerSlice({
            name,
            getState: get,
            setState: set,
        });
    }
    const use = () => {
        const [state, setState] = useState(get());
        useEffect(() => {
            const callback = () => {
                console.log(`🔄 Triggering re-render for slice: ${name}`);
                setState(get()); // 🔥 Ensure state updates correctly
            };
            if (!subscribers[name].has(callback)) {
                subscribers[name].add(callback);
            }
            return () => {
                subscribers[name].delete(callback);
            };
        }, []);
        return [state, set];
    };
    const select = (selector) => {
        return selector(get());
    };
    // Register the slice to the StoreManager if enabled
    if (registerToGlobalStore) {
        storeManager.registerSlice({
            name,
            getState: get,
            setState: set,
        });
    }
    return { get, set, use, select };
}
// 🛠️ Async Action Helper
// src/core.ts
export function createAsyncAction(asyncFn, { onPending, onFulfilled, onRejected, }) {
    return (set, payload) => __awaiter(this, void 0, void 0, function* () {
        try {
            set((state) => {
                state.loading = true;
                state.error = null;
                if (onPending)
                    onPending(state);
            });
            const result = yield asyncFn(payload);
            set((state) => {
                state.loading = false;
                if (onFulfilled)
                    onFulfilled(state, result);
            });
        }
        catch (error) {
            set((state) => {
                state.loading = false;
                state.error = (error === null || error === void 0 ? void 0 : error.message) || "Unknown Error";
                if (onRejected)
                    onRejected(state, error);
            });
        }
    });
}
// 🛠️ Enhanced Slice State Management with Redux DevTools Support
export function createSliceState({ 
// 🔥 Enforce T as an object
name, initialState, actions, asyncActions, middlewares = [], registerToGlobalStore = true, globalUpdate = true, }) {
    var _a, _b;
    if (store[name])
        throw new Error(`State slice "${name}" already exists.`);
    store[name] = initialState;
    subscribers[name] = new Set();
    const combinedMiddlewares = [defaultLogger, defaultAsync, ...middlewares];
    const devTools = typeof window !== "undefined" &&
        ((_b = (_a = window.__REDUX_DEVTOOLS_EXTENSION__) === null || _a === void 0 ? void 0 : _a.connect) === null || _b === void 0 ? void 0 : _b.call(_a));
    if (devTools) {
        devTools.init(initialState);
        console.log(`🛠️ Redux DevTools initialized for slice: "${name}"`);
    }
    const applyMiddleware = (updater, actionType = "STATE_UPDATE") => {
        let index = 0;
        const next = (nextUpdater) => {
            if (index < combinedMiddlewares.length) {
                const currentMiddleware = combinedMiddlewares[index++];
                currentMiddleware(store[name], nextUpdater, next);
            }
            else {
                console.log(`🛠️ Applying final state update for ${name}`);
                // ✅ Ensure that the state object is replaced
                const prevState = Object.assign({}, store[name]); // Clone previous state
                nextUpdater(store[name]); // Apply update
                // ✅ Ensure React recognizes state change
                store[name] = Object.assign({}, store[name]); // 🔥 Force object replacement
                // ✅ Notify Redux DevTools
                if (devTools) {
                    devTools.send(`${name}/${actionType}`, store[name]);
                }
                // ✅ Notify subscribers
                console.log(`🔔 Notifying subscribers of ${name}`);
                subscribers[name].forEach((callback) => callback());
                // ✅ Ensure global store updates
                storeManager.updateGlobalState();
            }
        };
        next(updater);
    };
    const set = (updater, actionType = "STATE_UPDATE") => {
        const prevState = Object.assign({}, store[name]);
        applyMiddleware((state) => {
            const newState = Object.assign({}, state);
            updater(newState);
            if (JSON.stringify(prevState) !== JSON.stringify(newState)) {
                console.log(`🛠️ State change detected for ${name}`);
                Object.assign(state, newState); // ✅ Ensure T is an object before applying changes
            }
        }, actionType);
        storeManager.updateGlobalState(); // 🔥 Ensure state propagates properly
    };
    const get = () => store[name];
    const use = () => {
        const [, forceRender] = useState(0);
        useEffect(() => {
            const callback = () => forceRender((prev) => prev + 1);
            subscribers[name].add(callback);
            return () => {
                subscribers[name].delete(callback);
            };
        }, []);
        return [get(), set];
    };
    const select = (selector) => {
        const [selectedState, setSelectedState] = useState(selector(get()));
        useEffect(() => {
            const callback = () => {
                setSelectedState(selector(get()));
            };
            subscribers[name].add(callback);
            return () => {
                subscribers[name].delete(callback);
            };
        }, [selector]);
        return selectedState;
    };
    const syncActions = {};
    for (const key in initialState) {
        if (Object.prototype.hasOwnProperty.call(initialState, key)) {
            const actionName = `set${key.charAt(0).toUpperCase() + key.slice(1)}`;
            syncActions[actionName] = (payload) => {
                set((state) => {
                    state[key] = payload;
                }, actionName);
            };
        }
    }
    const asyncActionMap = {};
    if (asyncActions) {
        const map = {
            handleAction: (action, handler) => {
                asyncActionMap[action] = (payload) => __awaiter(this, void 0, void 0, function* () {
                    try {
                        yield handler(store[name], payload);
                        subscribers[name].forEach((callback) => callback());
                        // Automatically update the global store if enabled
                        if (globalUpdate && registerToGlobalStore) {
                            storeManager.updateGlobalState();
                        }
                    }
                    catch (error) {
                        console.error(`❌ Async Action "${action}" Failed:`, error);
                    }
                });
            },
        };
        asyncActions(map);
    }
    // Register slice to StoreManager if enabled
    if (registerToGlobalStore) {
        storeManager.registerSlice({
            name,
            getState: get,
            setState: set,
        });
    }
    return {
        get,
        set,
        use,
        select,
        actions: syncActions,
        asyncActions: asyncActionMap,
    };
}
