import { useState, useEffect } from "react";
import { storeManager } from "./utils/store-manager";

// 🛡️ Middleware Definition
type Middleware<T> = (
  state: T,
  updater: (state: T) => void,
  next: (updater: (state: T) => void) => void
) => void;

type Subscriber = () => void;

interface StateSlice<T> {
  get: () => T;
  set: (updater: (state: T) => void, actionType?: string) => void;
  use: () => [T, (updater: (state: T) => void, actionType?: string) => void];
  select: <R>(selector: (state: T) => R) => R;
}

// Global Store and Subscribers
const store: Record<string, any> = {};
const subscribers: Record<string, Set<Subscriber>> = {};

// Default Middleware
const defaultLogger: Middleware<any> = (state, updater, next) => {
  console.log("🔄 Previous State:", state);
  next(updater);
  console.log("✅ Updated State:", state);
};

const defaultAsync: Middleware<any> = async (state, updater, next) => {
  try {
    const result = updater(state);

    if (
      result !== undefined &&
      result !== null &&
      typeof result === "object" &&
      typeof (result as any).then === "function"
    ) {
      console.log("⏳ Async Action Started");
      await result;
      console.log("✅ Async Action Completed");
    }
    next(updater); // Ensure next is called once
  } catch (error) {
    console.error("❌ Async Middleware Error:", error);
    next(() => {}); // Safely move to the next middleware
  }
};

// 🛠️ Core State Management Function with Redux DevTools Support
export function createState<T extends Record<string, any>>( // 🔥 Enforce T as an object
  name: string,
  initialState: T,
  middlewares: Middleware<T>[] = [],
  registerToGlobalStore = true
): StateSlice<T> & { select: <R>(selector: (state: T) => R) => R } {
  if (store[name]) throw new Error(`State slice "${name}" already exists.`);
  store[name] = initialState;
  subscribers[name] = new Set();

  const combinedMiddlewares = [defaultLogger, defaultAsync, ...middlewares];

  // 🛠️ Redux DevTools Integration
  const devTools =
    typeof window !== "undefined" &&
    (window as any).__REDUX_DEVTOOLS_EXTENSION__?.connect?.();

  if (devTools) {
    devTools.init(initialState);
    console.log(`🛠️ Redux DevTools initialized for slice: "${name}"`);
  }

  const applyMiddleware = (
    updater: (state: T) => void,
    actionType: string = "STATE_UPDATE"
  ) => {
    let index = 0;

    const next = (nextUpdater: (state: T) => void) => {
      if (index < combinedMiddlewares.length) {
        const currentMiddleware = combinedMiddlewares[index++];
        currentMiddleware(store[name], nextUpdater, next);
      } else {
        // Final updater application
        nextUpdater(store[name]);

        // Notify Redux DevTools
        if (devTools) {
          devTools.send(`${name}/${actionType}`, store[name]);
        }

        // Notify subscribers only once
        const seenCallbacks = new Set<Subscriber>();
        subscribers[name].forEach((callback) => {
          if (!seenCallbacks.has(callback)) {
            seenCallbacks.add(callback);
            callback();
          }
        });
      }
    };

    next(updater);
  };

  const get = () => store[name];

  // const set = (
  //   updater: (state: T) => void,
  //   actionType: string = "STATE_UPDATE"
  // ) => {
  //   console.log(`🛠️ set called with actionType: ${actionType}`);
  //   const prevState = { ...store[name] };
  //   applyMiddleware(updater, actionType);

  //   // Update global state and notify global subscribers
  //   if (registerToGlobalStore) {
  //     storeManager.registerSlice({
  //       name,
  //       getState: get,
  //       setState: set,
  //     });
  //   }

  //   console.log(`🔄 Previous State:`, prevState);
  //   console.log(`✅ Updated State:`, store[name]);
  // };
  const set = (
    updater: (state: T) => void,
    actionType: string = "STATE_UPDATE"
  ) => {
    const prevState = { ...store[name] };

    applyMiddleware((state) => {
      const newState = { ...state };
      updater(newState);

      if (JSON.stringify(prevState) !== JSON.stringify(newState)) {
        console.log(`🛠️ State change detected for ${name}`);
        Object.assign(state, newState); // 🔥 Correctly apply state updates
      }
    }, actionType);

    storeManager.updateGlobalState(); // 🔥 Ensure state propagates properly
  };

  // Register the slice to the StoreManager if enabled (only once during initialization)
  if (registerToGlobalStore) {
    storeManager.registerSlice({
      name,
      getState: get,
      setState: set,
    });
  }

  const use = (): [
    T,
    (updater: (state: T) => void, actionType?: string) => void
  ] => {
    const [state, setState] = useState(get());

    useEffect(() => {
      const callback = () => {
        console.log(`🔄 Triggering re-render for slice: ${name}`);
        setState(get()); // 🔥 Ensure state updates correctly
      };

      if (!subscribers[name].has(callback)) {
        subscribers[name].add(callback);
      }

      return () => {
        subscribers[name].delete(callback);
      };
    }, []);

    return [state, set];
  };

  const select = <R>(selector: (state: T) => R): R => {
    return selector(get());
  };

  // Register the slice to the StoreManager if enabled
  if (registerToGlobalStore) {
    storeManager.registerSlice({
      name,
      getState: get,
      setState: set,
    });
  }

  return { get, set, use, select };
}

// 🛠️ Async Action Helper
// src/core.ts

export function createAsyncAction<T, R>(
  asyncFn: (payload?: T) => Promise<R>,
  {
    onPending,
    onFulfilled,
    onRejected,
  }: {
    onPending?: (state: any) => void;
    onFulfilled?: (state: any, payload: R) => void;
    onRejected?: (state: any, error: any) => void;
  }
) {
  return async (set: (updater: (state: any) => void) => void, payload?: T) => {
    try {
      set((state) => {
        state.loading = true;
        state.error = null;
        if (onPending) onPending(state);
      });
      const result = await asyncFn(payload);
      set((state) => {
        state.loading = false;
        if (onFulfilled) onFulfilled(state, result);
      });
    } catch (error: any) {
      set((state) => {
        state.loading = false;
        state.error = error?.message || "Unknown Error";
        if (onRejected) onRejected(state, error);
      });
    }
  };
}

// 🛠️ Enhanced Slice State Management
type ActionHandlers<T> = {
  [key: string]: (state: T, payload?: any) => void;
};

type AsyncHandlers<T> = {
  [key: string]: (payload?: any) => Promise<void>;
};

// 🛠️ Enhanced Slice State Management with Redux DevTools Support
export function createSliceState<T extends Record<string, any>>({
  // 🔥 Enforce T as an object
  name,
  initialState,
  actions,
  asyncActions,
  middlewares = [],
  registerToGlobalStore = true,
  globalUpdate = true,
}: {
  name: string;
  initialState: T;
  actions?: Record<string, (state: T, payload?: any) => void>;
  asyncActions?: (actionMap: any) => void;
  middlewares?: Middleware<T>[];
  registerToGlobalStore?: boolean;
  globalUpdate?: boolean;
}) {
  if (store[name]) throw new Error(`State slice "${name}" already exists.`);
  store[name] = initialState;
  subscribers[name] = new Set();

  const combinedMiddlewares = [defaultLogger, defaultAsync, ...middlewares];

  const devTools =
    typeof window !== "undefined" &&
    (window as any).__REDUX_DEVTOOLS_EXTENSION__?.connect?.();

  if (devTools) {
    devTools.init(initialState);
    console.log(`🛠️ Redux DevTools initialized for slice: "${name}"`);
  }

  const applyMiddleware = (
    updater: (state: T) => void,
    actionType: string = "STATE_UPDATE"
  ) => {
    let index = 0;

    const next = (nextUpdater: (state: T) => void) => {
      if (index < combinedMiddlewares.length) {
        const currentMiddleware = combinedMiddlewares[index++];
        currentMiddleware(store[name], nextUpdater, next);
      } else {
        console.log(`🛠️ Applying final state update for ${name}`);

        // ✅ Ensure that the state object is replaced
        const prevState = { ...store[name] }; // Clone previous state
        nextUpdater(store[name]); // Apply update

        // ✅ Ensure React recognizes state change
        store[name] = { ...store[name] }; // 🔥 Force object replacement

        // ✅ Notify Redux DevTools
        if (devTools) {
          devTools.send(`${name}/${actionType}`, store[name]);
        }

        // ✅ Notify subscribers
        console.log(`🔔 Notifying subscribers of ${name}`);
        subscribers[name].forEach((callback) => callback());

        // ✅ Ensure global store updates
        storeManager.updateGlobalState();
      }
    };

    next(updater);
  };

  const set = (
    updater: (state: T) => void,
    actionType: string = "STATE_UPDATE"
  ) => {
    const prevState = { ...store[name] };

    applyMiddleware((state) => {
      const newState = { ...state };
      updater(newState);

      if (JSON.stringify(prevState) !== JSON.stringify(newState)) {
        console.log(`🛠️ State change detected for ${name}`);
        Object.assign(state as Record<string, any>, newState); // ✅ Ensure T is an object before applying changes
      }
    }, actionType);

    storeManager.updateGlobalState(); // 🔥 Ensure state propagates properly
  };

  const get = () => store[name];

  const use = (): [T, typeof set] => {
    const [, forceRender] = useState(0);

    useEffect(() => {
      const callback = () => forceRender((prev) => prev + 1);
      subscribers[name].add(callback);

      return () => {
        subscribers[name].delete(callback);
      };
    }, []);

    return [get(), set];
  };

  const select = <R>(selector: (state: T) => R): R => {
    const [selectedState, setSelectedState] = useState<R>(selector(get()));

    useEffect(() => {
      const callback = () => {
        setSelectedState(selector(get()));
      };
      subscribers[name].add(callback);

      return () => {
        subscribers[name].delete(callback);
      };
    }, [selector]);

    return selectedState;
  };

  const syncActions: Record<string, Function> = {};
  for (const key in initialState) {
    if (Object.prototype.hasOwnProperty.call(initialState, key)) {
      const actionName = `set${key.charAt(0).toUpperCase() + key.slice(1)}`;
      syncActions[actionName] = (payload: any) => {
        set((state) => {
          (state as any)[key] = payload;
        }, actionName);
      };
    }
  }

  const asyncActionMap: AsyncHandlers<T> = {};
  if (asyncActions) {
    const map: any = {
      handleAction: (
        action: string,
        handler: (state: T, payload?: any) => void
      ) => {
        asyncActionMap[action] = async (payload?: any) => {
          try {
            await handler(store[name], payload);
            subscribers[name].forEach((callback) => callback());

            // Automatically update the global store if enabled
            if (globalUpdate && registerToGlobalStore) {
              storeManager.updateGlobalState();
            }
          } catch (error) {
            console.error(`❌ Async Action "${action}" Failed:`, error);
          }
        };
      },
    };
    asyncActions(map);
  }

  // Register slice to StoreManager if enabled
  if (registerToGlobalStore) {
    storeManager.registerSlice({
      name,
      getState: get,
      setState: set,
    });
  }

  return {
    get,
    set,
    use,
    select,
    actions: syncActions,
    asyncActions: asyncActionMap,
  };
}
