type Slice<T> = {
  name: string;
  getState: () => T;
  setState: (updater: (state: T) => void) => void;
};

type GlobalState = Record<string, any>;

class StoreManager {
  private globalState: GlobalState = {};
  private globalSubscribers: Set<Function> = new Set();

  // Register a slice to the global store
  registerSlice<T>(slice: Slice<T>) {
    if (this.globalState[slice.name]) {
      console.warn(`Slice "${slice.name}" is already registered. Skipping.`);
      return;
    }

    this.globalState[slice.name] = slice.getState();

    // Wrap the slice's setState method to update the global state
    const originalSetState = slice.setState;
    slice.setState = (updater: (state: T) => void) => {
      originalSetState(updater); // Update the slice state
      this.globalState[slice.name] = slice.getState(); // Update global state
      this.notifySubscribers(); // Notify global subscribers
    };
  }

  // Get the global state
  getState() {
    return this.globalState;
  }

  // Subscribe to global state changes
  subscribe(callback: Function) {
    this.globalSubscribers.add(callback);
    return () => this.globalSubscribers.delete(callback);
  }

  // Public method to update global state and notify subscribers
  updateGlobalState() {
    this.notifySubscribers();
  }

  // Notify all global subscribers of changes (private)
  private notifySubscribers() {
    this.globalSubscribers.forEach((callback) => callback());
  }
}

export const storeManager = new StoreManager();
